/* INTERFACE TO ORVILLE LIB_COMMON FOR NTALKD */

#include "config.h"
#include <sys/stat.h>
#include "orville.h"
#include <pwd.h>

struct wrthdr wt_head;
int wstream;

extern char hostname[];

int orville_init(char *argv0)
{

	progname= leafname(argv0);
        readconfig(NULL);
        if (init_wstream(O_RDONLY))
        {
                syslog(LOG_ERR,"init_wstream failed");
                _exit(1);
        }
}


int ishisexception(int yesfile, const char *login)
{
struct passwd *hispwd;
#define LSS 1024
char buf[LSS+1];
char *b, *e;
FILE *fp;

    if ((hispwd= getpwnam(login)) == NULL) return 0;
    sprintf(buf, "%s/%s", hispwd->pw_dir, yesfile ? ".yeswrite" : ".nowrite");

    if ((fp= fopen(buf,"r")) == NULL) return 0;

    buf[LSS]= '\0';
    while (fgets(buf,LSS,fp) != NULL)
    {
    	if (buf[0] == '#') continue;
	e= buf-1;
	for (;;)
	{
	    for (b= e+1;
	    	*b!='\0' && strchr(" ,;:\t\n\r",*b)!=NULL; b++)
		    ;
	    if (*b == '\0') break;

	    /* Find next blank or punctuation character */
	    for (e= b+1;
	         *e!='\0' && strchr(" ,;:\t\n\r",*e)==NULL; e++)
		    ;
	    *e= '\0';

	    if (!strcmp(b,login))
	    {
	    	fclose(fp);
		return 1;
	    }
	}
    }
    fclose(fp);
    return 0;
}


/* MAYTALK - May we talk to this person?  Give the short tty name, and the
 * login time from utmp, and the stat buffer from stating the tty device.
 * Returns SUCCESS, PERMISSION_DENIED, or FAILURE.
 */

int maytalk(char *his_tty, time_t his_time, struct stat *his_st,
	const char *my_host, char *my_login, int *silent)
{
    struct wrttmp hiswrt;
    long hispos;

    /* Get his wrttmp */
    find_wrttmp(his_tty, his_time, &hiswrt, &hispos);

    /*
    syslog(LOG_DEBUG,"line=%s telpref=%d modepref=%c bells=%c except=%c",
    	hiswrt.wrt_line, hiswrt.wrt_telpref, hiswrt.wrt_modepref,
    	hiswrt.wrt_bells, hiswrt.wrt_except);
    */

    *silent= (hiswrt.wrt_bells == 'n');

    /* Does he refuse all talks? */
    if (!(hiswrt.wrt_telpref & TELPREF_TALK))
	return(PERMISSION_DENIED);

    /* Is he writing me?  (and I mean writing, not talking) */
    if (!strcmp(hostname,my_host) && !strcmp(hiswrt.wrt_what, my_login))
    	return(SUCCESS);

    /* Orville write allows root to write anyone, but it's hard to tell if the
     * talker is root, so we omit that feature.  Roots use write.
     */

    /* Are his permissions on? */
    if (
#ifndef TTYPERMS
        (hiswrt.wrt_mesg != 'n') ||
#endif
	(his_st->st_mode&020))
    {
    	/* Permissions are on, but are we an exception? */
	if (!f_exceptions || hiswrt.wrt_except != 'y' ||
	    !ishisexception(0,my_login))
	    return(SUCCESS);
    }
    else
    {
    	/* Permissions are off, but are we an exception? */
	if (f_exceptions && hiswrt.wrt_except == 'y' &&
	    ishisexception(1,my_login))
	    return(SUCCESS);
    }

    /* No support yet for 'talk help' */

    return(PERMISSION_DENIED);
}

#ifndef HAVE_GETUTENT
/* We never call this - it's just a dummy to keep the linker happy */
struct utmp *getutline(struct utmp *in) { }
#endif /* GETUTENT */
